#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <iomanip>
#include <iostream>
#include <map>
#include <vector>

#include "Message.h"
#include "ONSClientException.h"
#include "ONSFactory.h"

using namespace ons;

void printMessage(const vector<Message>& msgFoundList) {
  std::vector<Message>::const_iterator it = msgFoundList.begin();

  for (it; it != msgFoundList.end(); ++it) {
    std::cout << (*it).getBody() << std::endl;
  }
}

int main(int argc, char* argv[]) {
  ONSFactoryProperty factoryInfo;
  factoryInfo.setFactoryProperty(ONSFactoryProperty::ProducerId, "GID_xxxxxxxxxxxxx");
  factoryInfo.setFactoryProperty(ONSFactoryProperty::ConsumerId, "GID_xxxxxxxxxxxxx");
  factoryInfo.setFactoryProperty(ONSFactoryProperty::PublishTopics, "xxxxxxxxxxxx");
  factoryInfo.setFactoryProperty(ONSFactoryProperty::AccessKey, "xxxxxxxxxxxxx");
  factoryInfo.setFactoryProperty(ONSFactoryProperty::SecretKey, "xxxxxxxxxxxxx");
  //factoryInfo.setFactoryProperty(ONSFactoryProperty::ONSAddr, "http://xxx");  // ONS access point URL
  factoryInfo.setFactoryProperty(ONSFactoryProperty::NAMESRV_ADDR, "http://xxx");  // new ONS access point URL
  factoryInfo.setFactoryProperty(ONSFactoryProperty::MessageModel, ONSFactoryProperty::CLUSTERING);

  std::cout << "ConsumerId: " << factoryInfo.getConsumerId() << endl
            << "ProducerId: " << factoryInfo.getProducerId() << endl
            << "PublishTopics: " << factoryInfo.getPublishTopics() << endl
            << "AccessKey: " << factoryInfo.getAccessKey() << endl
            << "SecretKey: " << factoryInfo.getSecretKey() << endl;

  Producer* producer = ONSFactory::getInstance()->createProducer(factoryInfo);
  producer->start();

  PullConsumer* pullConsumer = ONSFactory::getInstance()->createPullConsumer(factoryInfo);

  pullConsumer->start();

  Message msg(factoryInfo.getPublishTopics(),  // topic
              "*",                             // tag
              "test"                           // body
  );

  // 先获取全部的队列
  std::vector<MessageQueueONS> mqs;
  try {
    pullConsumer->fetchSubscribeMessageQueues(factoryInfo.getPublishTopics(), mqs);
    std::cout << "fetch subscribe Message Queues success" << std::endl;
  } catch (ONSClientException& e) {
    std::cout << e.what() << std::endl;
  }

  for (int i = 0; i < 1000000; ++i) {
    try {
      // 往指定消息队列中发送消息
      int index = i % mqs.size();
      SendResultONS sendResult = producer->send(msg, mqs[index]);

    } catch (ONSClientException& e) {
      std::cout << "send messaage error" << e.what() << std::endl;
    }
  }

  std::vector<MessageQueueONS>::iterator iter = mqs.begin();
  for (; iter != mqs.end(); ++iter) {
    MessageQueueONS mq = (*iter);
    bool noNewMsg = false;
    do {
      try {
        // false 表示从内存中的offset table中获取offset
        // true 表示从远端的服务器中获取最后消费的offset
        long long offset = pullConsumer->fetchConsumeOffset(mq, false);
        if (offset == -1) offset = 0;

        PullResultONS result = pullConsumer->pull(mq, "*", offset, 32);

        // if pull request timeout or received NULL response, pullStatus will be
        // set to BROKER_TIMEOUT,
        // And nextBeginOffset/minOffset/MaxOffset will be set to 0
        if (result.pullStatus != ONS_BROKER_TIMEOUT) {
          // 更新内存中的offset table
          pullConsumer->updateConsumeOffset(mq, result.nextBeginOffset);
          // 更新服务器上的消费进度，这个需要定时做，要更新的mq必须是保存在内存中的offset table中
          pullConsumer->persistConsumerOffset4PullConsumer(mq);
        } else {
          cout << "broker timeout occur" << endl;
        }
        switch (result.pullStatus) {
          case ONS_FOUND:
            printf(
                "find consumer msg, its topic is:%s, its broker name is:%s, "
                "its queueId is:%d",
                mq.getTopic().c_str(), mq.getBrokerName().c_str(), mq.getQueueId());

            printMessage(result.msgFoundList);
          case ONS_NO_MATCHED_MSG:
          case ONS_OFFSET_ILLEGAL:
          case ONS_BROKER_TIMEOUT:
            break;
          case ONS_NO_NEW_MSG:
            noNewMsg = true;
            break;
          default:
            break;
        }
      } catch (ONSClientException& e) {
        std::cout << e.what() << std::endl;
      }
    } while (!noNewMsg);
  }

  // While
  producer->shutdown();
  pullConsumer->shutdown();

  return 0;
}
